@@ -11,6 +11,7 @@ use SVK::Util qw( abs2rel );
 sub options {
     ("q|quiet"    => 'quiet',
      "no-ignore"  => 'no_ignore',
+     "v|verbose"  => 'verbose',
     );
 }
 
@@ -20,44 +21,79 @@ sub parse_arg {
     return $self->arg_condensed (@arg);
 }
 
+sub flush_print {
+    my ($root, $target, $entry, $status, $baserev, $from_path, $from_rev) = @_;
+    my ($crev, $author);
+    my $fs = $root->fs;
+    if ($from_path) {
+	$from_path =~ s{^file://\Q$target->{repospath}\E}{};
+        $crev = $fs->revision_root ($from_rev)->node_created_rev ($from_path);
+	$author = $fs->revision_prop ($crev, 'svn:author');
+	$baserev = '-';
+    } elsif ($status->[0] =~ '[I?]') {
+	$baserev = '';
+	$crev = '';
+	$author = ' ';
+    } elsif ($status->[0] eq 'A') {
+	$baserev = 0;
+    } elsif ($status->[0] !~ '[!~]') {
+        my $p = $target->{path};
+	my $path = $p eq '/' ? "/$entry" : (length $entry ? "$p/$entry" : $p);
+	$crev = $root->node_created_rev ($path);
+	$author = $fs->revision_prop ($crev, 'svn:author');
+    }
+
+    my $report = $target->{report};
+    my $newentry = length $entry
+	? SVK::Target->copath ($report, $entry)
+	: SVK::Target->copath ('', length $report ? $report : '.');
+    no warnings 'uninitialized';
+    print sprintf ("%1s%1s%1s %8s %8s %-12.12s \%s\n", @{$status}[0..2],
+                   defined $baserev ? $baserev : '? ',
+		   defined $crev ? $crev : '? ',
+		   $author ? $author : ' ?',
+                   $newentry);
+}
+
 sub run {
     my ($self, $target) = @_;
     my $xdroot = $self->{xd}->xdroot (%$target);
     my $report = $target->{report};
     $report .= '/' if $report;
+    my $editor = SVK::Editor::Status->new (
+	  report => $target->{report},
+	  ignore_absent => $self->{quiet},
+	  $self->{verbose} ?
+	  (notify => SVK::Notify->new (
+	       flush_baserev => 1,
+	       flush_unchanged => 1,
+	       cb_flush => sub { flush_print ($xdroot, $target, @_); }
+	   )
+	  )                :
+	  (notify => SVK::Notify->new_with_report ($target->{report},
+		undef, 1)
+	  )
+      );
     $self->{xd}->checkout_delta
 	( %$target,
 	  xdroot => $xdroot,
 	  nodelay => 1,
 	  delete_verbose => 1,
-	  editor => SVK::Editor::Status->new (
-	      report => $target->{report},
-              ignore_absent => $self->{quiet},
-	      notify => SVK::Notify->new_with_report ($target->{report}, undef, 1)
-	  ),
+	  editor => $editor,
 	  cb_conflict => \&SVK::Editor::Status::conflict,
 	  cb_obstruct => \&SVK::Editor::Status::obstruct,
+	  $self->{verbose} ?
+	      (cb_unchanged => \&SVK::Editor::Status::unchanged
+	      )            :
+	      (),
 	  $self->{recursive} ? () : (depth => 1),
 	  $self->{no_ignore} ?
-              (cb_ignored =>
-                 sub { my $path = abs2rel($_[1],
-                                          $target->{copath} => 
-                                            length($report) ? $report : undef
-                                         );
-                       print "I   $path\n"
-                 }
+              (cb_ignored => \&SVK::Editor::Status::ignored
               )              :
               (),
 	  $self->{quiet} ?
               ()         :
-              (cb_unknown =>
-                 sub { my $path = abs2rel($_[1],
-                                          $target->{copath} => 
-                                            length($report) ? $report : undef
-                                         );
-                       print "?   $path\n"
-                 }
-              )
+              (cb_unknown => \&SVK::Editor::Status::unknown)
 	);
     return;
 }
@@ -79,6 +115,7 @@ SVK::Command::Status - Display the status of items in the checkout copy
  -q [--quiet]           : do not display files not under version control
  --no-ignore            : disregard default and svn:ignore property ignores
  -N [--non-recursive]   : do not descend recursively
+ -v [--verbose]         : print full revision information on every item
 
 =head1 DESCRIPTION
 
@@ -110,8 +147,10 @@ Third column, scheduled commit will contain addition-with-history
   ' ' no history scheduled with commit
   '+' history scheduled with commit
 
-The working copy path, which starts in the fourth column is always
-the final field, so it can include spaces.
+Remaining fields are variable width and delimited by spaces:
+  The working revision (with -v)
+  The last committed revision and last committed author (with -v)
+  The working copy path is always the final field, so it can include spaces.
 
 Example output:
 
@@ -119,6 +158,13 @@ Example output:
    M  bar.c
   A + qax.c
 
+  svk status --verbose wc
+   M        53        2 sally        wc/bar.c
+            53       51 harry        wc/foo.c
+  A +        -       ?   ?           wc/qax.c
+            53       43 harry        wc/zig.c
+            53       20 sally        wc
+
 =head1 AUTHORS
 
 Chia-liang Kao E<lt>clkao@clkao.orgE<gt>
