@@ -1,6 +1,6 @@
 package SVK::Notify;
 use SVK::I18N;
-use SVK::Util qw( abs2rel $SEP to_native from_native get_encoding);
+use SVK::Util qw( abs2rel $SEP to_native get_encoding);
 use strict;
 
 =head1 NAME
@@ -43,12 +43,10 @@ sub print_report {
     # XXX: $report should already be in native encoding, so this is wrong
     my $print_native = $enc->name eq 'utf8'
 	? $print
-	: sub { to_native($_[0], 'path', $enc);
+	: sub { to_native ($_[0]);
 		goto \&$print;
 	    };
     return $print_native unless defined $report;
-    $report = "$report";
-    from_native($report, 'path', $enc);
     sub {
 	my $path = shift;
 	if ($target) {
@@ -60,12 +58,12 @@ sub print_report {
 	    }
 	}
 	if (length $path) {
-	    $print_native->($is_copath ? SVK::Path::Checkout->copath ($report, $path)
+	    $print_native->($is_copath ? SVK::Target->copath ($report, $path)
 			               : length $report ? "$report/$path" : $path, @_);
 	}
 	else {
 	    my $r = length $report ? $report : '.';
-	    $print_native->($is_copath ? SVK::Path::Checkout->copath('', $r) : $r,
+	    $print_native->($is_copath ? SVK::Target->copath('', $r) : $r,
 			    @_);
 	}
     };
@@ -100,7 +98,6 @@ sub notify_translate {
 
 sub node_status {
     my ($self, $path, $s) = @_;
-    Carp::cluck unless defined $path;
     $self->{status}{$path}[0] = $s if defined $s;
     return exists $self->{status}{$path} ? $self->{status}{$path}[0] : undef;
 }
